﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace HeatmapTest
{
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName]string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        private List<MySeries> intensityGridSeries
            = new List<MySeries>();
        public List<MySeries> IntensityGridSeries
        {
            get { return intensityGridSeries; }
            set
            {
                intensityGridSeries = value;
                NotifyPropertyChanged();
            }
        }

        private bool isUpdateChart = false;
        public bool IsUpdateChart
        {
            get { return isUpdateChart; }
            set
            {
                isUpdateChart = value;
                NotifyPropertyChanged();
            }
        }

        DispatcherTimer m_timer;

        const int DataColumnCount = 2000;
        const int DataRowCount = 2000;
        double[][] m_data = null;
        int m_iRound = 0;
        private DateTime startTime;

        public MainWindowViewModel()
        {
            var mSe = new MySeries();
            if (m_data == null)
            {
                //Create a data array
                m_data = new double[DataColumnCount][];
                for (int iCol = 0; iCol < DataColumnCount; iCol++)
                {
                    m_data[iCol] = new double[DataRowCount];
                }
                //IntensityGridSeries[0].Data = ConvertIntesityPoint(m_data);
                mSe.Data = m_data;
            }
            this.IntensityGridSeries.Add(mSe);

            m_timer = new DispatcherTimer();
            m_timer.Interval = TimeSpan.FromMilliseconds(1);
            m_timer.Tick += new EventHandler(m_timer_Tick);
        }

        //Call from View
        public MyDelegateCommand StartCommand => new MyDelegateCommand(() => Start());
        public void Start()
        {
            m_timer.Start();
            startTime = DateTime.Now;
        }

        void m_timer_Tick(object sender, EventArgs e)
        {
            IsUpdateChart = true;

            //Set new values to data array
            for (int iCol = 0; iCol < DataColumnCount; iCol++)
            {
                for (int iRow = 0; iRow < DataRowCount; iRow++)
                {
                    //This change is not reflect in view.
                    IntensityGridSeries[0].Data[iCol][iRow] = (double)m_iRound + 20.0 * Math.Sin((double)iCol * (double)iRow / 5000.0);
                }
            }

            //Call View Method
            IsUpdateChart = false;

            m_iRound++;
            if (m_iRound == 50)
            {
                Debug.WriteLine($"info Round = {m_iRound}, time = {DateTime.Now - startTime}");
                m_iRound = 0;
            }
        }
    }
}
